/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "DataFlow.h"
using namespace std;

//const static public class member
const string DataFlow::headerOverheadString = "\n--------------!!! WARNING !!!--------------\n WARNING : THIS FILE IS TOO BIG,\n\t ";
const string DataFlow::footerOverheadString = " BYTES WAS REMOVED          \n-------------------------------------------\n";
size_t DataFlow::bufferFlowMax = DataFlow::DEFAULT_BUFFER_FLOW_MAX;

DataFlow::DataFlow(size_t flowSize) {
	// init current buffer
	posFirstBuffer = 0;
	posSecondBuffer = 0;
	overload = false;
	if(flowSize > 0) bufferFlowMax = flowSize;
}

size_t DataFlow::fillFrom ( FILE* fd ) {
	char *firstBuffer = new char[bufferFlowMax+1];
	char *secondBuffer = new char[bufferFlowMax+1];
	size_t nbBytesRead;
	
	assert(fd != NULL);
	
	//ends final buffers
	memset(firstBuffer, '\0', bufferFlowMax+1);
	memset(secondBuffer, '\0', bufferFlowMax+1);
	
	//read up to BUFFER_FLOW_MAX bytes
	nbBytesRead = safeRead(fd, firstBuffer, bufferFlowMax);
	posFirstBuffer = nbBytesRead;
	/* if first reading isn't enough */
	if(nbBytesRead == bufferFlowMax){
		/* the next reading erase the previous */
		do{
			nbBytesRead = safeRead(fd, secondBuffer, bufferFlowMax);
			posSecondBuffer += nbBytesRead;
		}
		while( nbBytesRead > 0);
		overload = ((long int)(posSecondBuffer - bufferFlowMax) > 0);
	}
	
	nbBytesRead = size();
	this->firstBuffer = firstBuffer;
	this->secondBuffer = secondBuffer;
	return nbBytesRead;
}

size_t DataFlow::fillSpecificMessage(std::string chain)
{
	posFirstBuffer = chain.size();
	firstBuffer = chain;
	return chain.size();
}


size_t DataFlow::createHash() const {
	return hash_fn(string(getContent()));
}

bool DataFlow::matchHash ( size_t hash ) const {
	return (createHash() == hash);
}

std::string DataFlow::getContent()  const {
	string insert, finalChain;
	int overhead, modPosSecBuf;

	overhead = posSecondBuffer - bufferFlowMax;
	finalChain = firstBuffer;
	
	//detect if second buffer have been overwritten
	if(overload){
		//adding header in ouput in case of overwritting
 		insert = headerOverheadString +  static_cast<ostringstream*>(&(ostringstream() << overhead))->str() + footerOverheadString;
		finalChain += insert;
		
		//split buffer to reorder correct output
		modPosSecBuf = posSecondBuffer%bufferFlowMax;
		finalChain += secondBuffer.substr( modPosSecBuf, secondBuffer.size() -1);
		finalChain += secondBuffer.substr(0, modPosSecBuf - 1);
	} else if(posSecondBuffer > 0){
		finalChain += secondBuffer.substr(0, posSecondBuffer - 1);
	}

	//return correct chain
	return(finalChain);
}

size_t DataFlow::size() const {
	return (overload) ? 2*bufferFlowMax : posFirstBuffer + posSecondBuffer;
}

DataFlow::~DataFlow() {

}

DataFlow& DataFlow::operator= ( const DataFlow& other ) {
	this->firstBuffer = other.firstBuffer;
	this->secondBuffer = other.secondBuffer;
	
	this->overload = other.overload;
	this->posFirstBuffer = other.posFirstBuffer;
	this->posSecondBuffer = other.posSecondBuffer;
	return *this;
}

DataFlow::DataFlow(const DataFlow& other)
{
	*this = other;
}
