/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "Dispatcher.h"
using namespace std;

Dispatcher::~Dispatcher() {
	//cleaners
	delete orchestra;
	delete resMan;
	delete jobsMan;
}

Dispatcher::Dispatcher ( Configuration* conf){
	assert(conf != NULL);
	config = conf;
	jobsMan = new JobManager(conf);
	assert(jobsMan != NULL);
	resMan = new ResourcesManager;
	assert(resMan != NULL);
}

void Dispatcher::setRunner()
{
	if(config->isMaster()){
		orchestra = new RunnerMaster(jobsMan, config);
	}
	else {
		orchestra = new RunnerSlave(jobsMan, config);
	}
	
	assert(orchestra != NULL);
}

void Dispatcher::startScheduling() {

	size_t nbCurrentLaunches = 0;
	Worker* cur = NULL;
	
	//pre actions
	assert(resMan != NULL);
	assert(orchestra != NULL);
	assert(jobsMan != NULL);

	resMan->setNbResources(config->system().getNbMaxResources());
	orchestra->pullJobsFromFiles();
	orchestra->startCompilation();
	
	/* THE IDEA HERE:
	 * Fill the available resources with max workload as we can according to 
	 * number of left jobs and max simultaneous slaves.
	 * When no slaves can be started, JCHRONOSS waits for the first result and apply some post-treatment
	 * Then, we try to reuse resources immediatly.
	 * 
	 * This steps are repeated until no jobs left
	 */
	
	//while some jobs haven't be executed or at least one slave are running
	while(!jobsMan->isEmpty() || nbCurrentLaunches > 0){
		orchestra->preActions();
		//while jobs are left AND (some slaves can be launched AND availables resources
 		while(!jobsMan->isEmpty() && (nbCurrentLaunches < config->system().getNbMaxSlaves() || config->system().getNbMaxSlaves() == SystemConfiguration::DEFAULT_SLAVES_VALUE) && resMan->getNbResources() > 0){

			//get best list to take jobs (list with biggest jobs)
			if(jobsMan->getHigherValidList(resMan->getNbResources()) < 0){
				break;
			}
			
			cur = new Worker;
			assert(cur != NULL);
			// fill the worker
			orchestra->fillWorker(cur, resMan->getNbResources());
			resMan->update((int)cur->getNbResources()*(-1));
			nbCurrentLaunches++;
			
			//start the worker
			orchestra->launchWorker(cur);
			
			//detach the worker
			cur = NULL;
		}
		//if some slaves are running
		if(nbCurrentLaunches > 0){
			//wait next slave end
			cur = orchestra->waitNextWorker();
			assert(cur != NULL);
			nbCurrentLaunches--;
			resMan->update(cur->getNbResources());
			
			//post treatments about results
			orchestra->postActions(cur);
			delete cur;
			cur = NULL;
		}
	}
	
	//push results in output files
	orchestra->pushJobsIntoFiles();
}


