/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef DISPATCHER_H
#define DISPATCHER_H

#include "utils.h"
#include "JobManager.h"
#include "RunnerMaster.h"
#include "RunnerSlave.h"
#include "ResourcesManager.h"
#include "HashTable.h"

/// Main class dispatching the work over the worker and master/slave
/**
 * The dispatcher contains all components needed to do the work. it's the orchestra chief.
 * It's contains a runner, which can be a master or a slave, allowing to identify in which
 * mode we are. It's from this object than all others stuffs will be scheduled.
 */
class Dispatcher {
private:
	/************** MEMBERS **************/
	JobManager* jobsMan;       ///< The job Manager containing jobs
	ResourcesManager* resMan;  ///< The Resources Manager containing resources states
	Configuration* config;     ///< The current global configuration
	Runner* orchestra;         ///< This object allows to distinct master or slave work
	HashTable workersTable;    ///< This references current running workers

public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///the only one constructor
	/**
	 * Used to set the dispatcher on application starting
	 * \param[in] conf The configuration set previously
	 */
	Dispatcher(Configuration* conf);
	/// Starting the application
	void startScheduling();
	/// Define the current mode (master/slave)
	void setRunner();
	/// Virtual destructor to unset the dispatcher
	virtual ~Dispatcher();
};

#endif // DISPATCHER_H
