/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "FileManager.h"
using namespace std;

//static counter to add unicity in file name
size_t FileManager::newFileID = 0;

FileManager::FileManager() {
}

FileManager::FileManager ( std::string* exeName, FileMode access) {
	char * name;
	
	//pre actions
	assert(exeName != NULL);
	
	name = new char[exeName->size()+1];
	assert(name != NULL);
	
	name[exeName->size()] = '\0';
	exeName->copy(name, exeName->size());
	
	this->name = new string(basename(name));
	this->path = new string(dirname(name));
	assert(this->name != NULL);
	assert(this->path != NULL);
	
	mode = access; 
	stream = NULL;
	delete [] name;
}

bool FileManager::close() {
	//flush & close
	stream->flush();
	stream->close();
	delete stream; stream = NULL;
	return stream == NULL;
}
bool FileManager::open() {
	switch(mode){
		case INPUT_ACCESS:
			stream = new fstream(toString().c_str(), ios::in);
			break;
		case OUTPUT_ACCESS:
			stream = new fstream(toString().c_str(), ios::out | ios::trunc);
			break;
		case INPUT_ACCESS_BIN:
			stream = new fstream(toString().c_str(), ios::in | ios::binary);
			break;
		case OUTPUT_ACCESS_BIN:
			stream = new fstream(toString().c_str(), ios::out | ios::trunc | ios::binary);
			break;		
	}
	assert(stream != NULL);
	return stream->is_open();
}

bool FileManager::isOpen() const {
	return stream->is_open();
}

bool FileManager::isCreated(std::string* file){
	if(file == NULL) return true;
	//check if file exists, trying to open it
	ifstream p((*file).c_str());
	return p;
}
FileManager::~FileManager() {
	delete name;
	delete path;
}

FileManager& FileManager::operator<< ( std::string chain ) {
	stream->write(chain.c_str(), chain.size());
	return *this;
}



bool FileManager::read(std::string* arg1, long unsigned int arg2) {
	char *argChar = new char[arg2+1];
	assert(arg1 != NULL);
	for(size_t i = 0; i < arg2+1; i++)
		argChar[i] = '\0';
	stream->read(argChar, arg2);
	*arg1 = argChar;
	delete [] argChar;
	return (!stream->eof());
}

bool FileManager::read(char* arg1, long unsigned int arg2) {
	assert(arg1 != NULL);
	stream->read(arg1, arg2);
	return (!stream->eof());
	
}

void FileManager::write(const char* obj, long unsigned int size) {
	assert(obj != NULL);
	stream->write(obj, size);
	stream->flush();
}

string FileManager::toString() const
{
	//pre_actions
	assert(path != NULL);
	assert(name != NULL);
	
	return ((*path)+"/"+(*name));
}

std::string FileManager::getPath() const
{
	assert(path != NULL);
	
	return ((*path)+"/");
}

string FileManager::getBaseName() const
{
	stringstream flux;
	
	assert(name != NULL);
	
	FileManager::newFileID++;
	flux << FileManager::newFileID << "-" << *name;
	return flux.str();
}

string FileManager::getName() const
{
	assert(name != NULL);
	
	return *name;
}
