/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "Filter.h"
using namespace std;

Filter::Filter() : 
	type(WHATEVER)
{}

Filter::Filter ( const Filter& other ) :
	type(other.type), rules(other.rules)
{}

Filter::~Filter() {
	for(vector<FilterRule*>::iterator it = rules.begin(); it != rules.end();){
		delete (*it);
		it = rules.erase(it);
	}
}

bool Filter::accept ( std::string name ) const {
	bool valid = false;
	if(type == WHATEVER) return true;
	
	//check if name matchs with at list on rule
	for(size_t i=0; i < rules.size() && !valid; i++){
		valid = (name.find(rules[i]->getName()) != string::npos);
	}
	
	if(type == BLACKLIST) valid = !valid;
	
	return valid;
}
void Filter::addRule ( FilterRule* rule ) {
	assert(rule != NULL);
	rules.push_back(rule);
}

bool Filter::fillFromFile ( std::string fileName, FilterMode mode ) {
	FilterRule* current;
	
	std::ifstream file(fileName.c_str(), ios::in);
	if(!file){
		printWarning("Unable to find the filter file "+fileName+" ! This filter will be ignored !");
		return false;
	}
	
	current = new FilterRule;
	assert(current != NULL);
	while(file >> *current){	
		addRule(current);
		current = new FilterRule;
		assert(current != NULL);
	}
	delete current;
	current = NULL;
	
	type = mode;
	file.close();
	return true;
}

void Filter::display() const {
	vector<FilterRule*>::const_iterator cur;
	string stype = (type == WHITELIST) ? "WHITELIST" : "BLACKLIST";
	cout << "\t- Mode = " << stype << endl;
	for(cur = rules.begin(); cur != rules.end(); cur++)
		cout << "\t\t- "<< (*cur)->getName() << endl;
}

