/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "FilterBox.h"
using namespace std;

FilterBox::FilterBox() : nbFilters(0) {}

FilterBox::FilterBox ( const FilterBox& other ) {
	nbFilters = other.nbFilters;
	filters = other.filters;
}

FilterBox::~FilterBox() {
	for(vector<Filter*>::iterator it = filters.begin(); it != filters.end();){
		delete (*it);
		it = filters.erase(it);
	}
	nbFilters=0;
}

bool FilterBox::accept ( std::string name, std::string pathName ) const {
	bool valid = true;
	vector<Filter*>::const_iterator current = filters.begin();

	while(valid && current != filters.end()){
		valid = (*current)->accept(pathName + "." +  name);
		current++;
	}
	return valid;
}
bool FilterBox::addFilter ( FileManager* file, FilterMode mode ) {
	Filter* new_filter = new Filter;
	bool ok;
	
	assert(file != NULL);
	assert(new_filter != NULL);
	
	ok = new_filter->fillFromFile(file->toString(), mode);
	if(ok){
		nbFilters++;
		filters.push_back(new_filter);
	} else {
		delete new_filter;
		new_filter = NULL;
	}
	
	return ok;	
}

void FilterBox::display() const {
	vector<Filter*>::const_iterator current;
	cout << "Filters:\n";
	for(current = filters.begin(); current != filters.end(); current++)
		(*current)->display();
	
}

