/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef FILTERBOX_H
#define FILTERBOX_H

#include "utils.h"
#include "Filter.h"
#include "FileManager.h"

///A filterBox gathers all filters (blacklist, whitelist) into one object
/**
 * A filterbox is msotly used like an array with a list and a size.
 */
class FilterBox {
private:
	/************** MEMBERS **************/
	int nbFilters;                ///< number of filters of application
	std::vector<Filter*> filters; ///< list of filters
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	FilterBox();
	///constructor by copy
	FilterBox ( const FilterBox& other );
	/// add a filter to the application
	/**
	 * add the given file as a new filter in database
	 * \param[in] file the file to upload
	 * \param[in] mode mode to apply to this filter
	 * \return <b>True</b> if succeded
	 * \return <b>False</b> otherwise
	 */
	bool addFilter(FileManager* file, FilterMode mode);
	/// virtual destructor to unset FilterBox
	virtual ~FilterBox();
	
	/****** CONST ******/
	///check if a job is a valid one
	/**
	 * Check if the job matches in the whole database of filters.
	 * \param[in] name the job name
	 * \param[in] pathName the job package to match
	 * \return <b>True</b> if job is valid
	 * \return <b>False</b> otherwise
	 */
	bool accept(std::string name, std::string pathName) const;
	/// display a filterBox (used to debug)
	void display() const;
};

#endif // FILTERBOX_H
