/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef FILTERRULE_H
#define FILTERRULE_H

#include "utils.h"

/// represents a most little unit of filtering in database
/**
 * A filter rule is the struct which allows to match a job with a constraint.
 * Here, a filter rule is constituted like Java class identification:
 * package.subpackage.subpackage.jobName. This path is re-constructed from job
 * path and stored in a rule. To ensure a job validity, we compare job path to the rule.
 */
class FilterRule {
private:
	/************** MEMBERS **************/
	std::string name;   ///< the path chain used to compare
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	explicit FilterRule();
	///Create a rule from a given string
	/**
	 * \param[in] rule the rule containing a job path
	 */
	FilterRule(std::string rule);
	/// read a rule from file
	/**
	 * \param[in] cin the file where read (need to be open)
	 * \param[in] obj the rule  to fill
	 * \return stream reference to concatenate instructions
	 */
	friend std::istream& operator>>(std::istream& cin, FilterRule& obj);
	/// set current rule with given name
	/**
	 * \param[in] name job path to set the current filter rule
	 */
	void setName(std::string name);
	
	/****** CONST ******/
	///rule getter
	/**
	 * \return the job path
	 */
	std::string getName() const;
};


#endif // FILTERRULE_H
