/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef HASHTABLE_H
#define HASHTABLE_H
#include "utils.h"
#include "Worker.h"
/// class gathering runnning worker into an hash table with tuple (pid, worker*)
/**
 * When a worker is launched, the master needs to wait the worker ends. it stores worker information
 * in this table. When worker ends, the master looks for matching worker in the table and fill jobs with
 * results found in binary file. This class just allows to manipulate the table.
 */
class HashTable {
private:
	/************** MEMBERS **************/
	///clas hashItem, which represents an entry in table
	/**
	 * An entry in table contains the process's pid where the worker is launched and and pointer
	 * on the worker. This class is a private class member, only the HashTable class should be
	 * able to manipulate it
	 */
	class HashItem {
	public:
		int pid;       ///< the pid for the matching launched worker
		Worker* work;  ///< a pointer to the worker
		/// construct a entry
		/**
		 * \param[in] p the pid
		 * \param[in] w the worker pointer
		 */
		HashItem(int p, Worker* w) : pid(p), work(w) {};
	};
	std::list<HashItem*> table;   ///< the workers table

	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// looks for a launched worker in the list from the given pid returned by wait
	/**
	 * \param[in] pid the pid attached to the worker when it have been launched
	 * \return <b>Iterator</b> on element if found
	 * \return <b>list.end() </b> otherwise
	 */
	std::list<HashItem*>::iterator find( int pid );
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// add an entry into the table
	/**
	 * Given arguments, a new entry is created in the table, the worker shouldn't be NULL
	 * \param[in] pid the pid attached to the worker
	 * \param[in] nw the worker pointer
	 */
	void add( int pid, Worker* nw );
	/// remove an entry according to the pid
	/**
	 * \param[in] pid the pid attached to the worker to remove
	 * \return <b>Worker pointer</b> if found in table
	 * \return <b>NULL</b> otherwise
	 */
	Worker* remove(int pid);
	///stop all running workers when a signal is caught
	void stopAll();

	/****** CONST ******/
	///check if table is empty (used for tests)
	/**
	 * \return <b>True</b> if list is empty
	 * \return <b>False</b> otherwise
	 */
	bool isEmpty() const;
};

#endif // HASHTABLE_H
