/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef JOBCONSTRAINT_H
#define JOBCONSTRAINT_H

#include "utils.h"

/// represents a job constraint applied on a job
/**
 * A Job constraint is a rule which is applied to some jobs. Jobs with same constraints
 * are grouped together and some actions can be started on this jobs group specifically
 * (e.g. compilations running, etc...)
 */
class JobConstraint {
private:
	/************** MEMBERS **************/
	std::string name;        ///< constraint name
	
public:
	/************** STATICS **************/
	/****** CONST ******/
	static const std::string COMPILATION_TAG; ///< defined known constraint : compilation
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	//construct a JobConstraint with a given name
	/**
	 * \param[in] name constraint name
	 */
	JobConstraint( std::string name );
	/****** CONST ******/
	/// check if current constraint belongs to the group defined by the parameter
	/**
	 * \param[in] group the group name used to check constraint matching
	 * \return <b>True</b> if constraint belongs to the same as given id
	 * \return <b>False</b> otherwise
	 */
	bool isBelongingTo(std::string group) const;
	/// operaator overloading: JobConstraint comparison
	/**
	 * check if two constraints are the same
	 * \param[in] other the constraint to compare with
	 * \return <b>True</b> if they are the same
	 * \return <b>False</b> otherwise
	 */
	bool operator==(const JobConstraint& other) const;
	/// get constraint name
	/**
	 * \return a string containing the constraint name
	 */
	std::string getName() const;
	/// print a jobConstraint on the screen
	void display() const;
};

#endif // JOBCONSTRAINT_H
