/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "JobResult.h"
using namespace std;

JobResult::JobResult() {
	this->header.hashedName = 0;
	this->header.hashedData = 0;
	this->header.executionTime = 0;
	this->header.magik = 0;
	this->header.sizeData = 0;
	this->header.finalReturnCode = 0;
	this->header.startingTime = 0;
}

void JobResult::fillHeader ( std::string jobName, size_t jobId, int rc,
			   double time, double begin ) {
	this->header.id = jobId;
	this->header.hashedName = hash_fn(jobName);
	this->header.finalReturnCode = rc;
	this->header.executionTime = time;
	this->header.startingTime = begin;
	this->header.magik = DEFAULT_MAGIK_NUMBER;
}

void JobResult::insertData ( std::string data ) {
	this->flow= data;
	this->header.hashedData = hash_fn(data);
	this->header.sizeData = data.size();
}

JobResult::~JobResult(){
}

size_t JobResult::getHashName() const
{
	return header.hashedName;
}

unsigned int JobResult::getMagik() const
{
	return header.magik;
}

void JobResult::display() const
{
	cout << "+-------------- JOB RESULT --------------+" << endl;
	cout << "| - Name Hash : " <<  header.hashedName << endl;
	cout << "| - RC : " << header.finalReturnCode << endl;
	cout << "| - magik : " << header.magik << endl;
	cout << "| - Time : " << header.executionTime << endl;
	cout << "| - Data : \"" << flow << "\"" <<endl;
	cout << "+----------------------------------------+" << endl;	
}

void JobResult::checksum() const
{
	assert(header.magik == DEFAULT_MAGIK_NUMBER);
	assert(hash_fn(flow) == header.hashedData);
}

int JobResult::getFinalRC() const
{
	return header.finalReturnCode;
}

bool JobResult::read(FileManager* file) {
	bool ok = true;
	CHECK(file->isOpen());
	
	ok = file->read(reinterpret_cast<char*>(&this->header), sizeof(JobResultHeader));
	if(!ok) return false;
	
	ok = file->read(&flow, this->header.sizeData);
	if(!ok) return false;
	return true;
}

void JobResult::write(FileManager* file) {

	CHECK(file->isOpen());
	file->write(reinterpret_cast<char*>(&this->header), sizeof(JobResultHeader));
	file->write(flow.c_str(), flow.size());
}

double JobResult::getTime() const
{
	return header.executionTime;
}

std::string JobResult::getData() const
{
	return flow;
}

double JobResult::getStartTime() const
{
	return header.startingTime;
}

size_t JobResult::getHashData() const
{
	return header.hashedData;
}
size_t JobResult::getSizeData() const
{
	return header.sizeData;
}

size_t JobResult::getId() const
{
	return header.id;
}


