/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef POLICY_H
#define POLICY_H
#include "JobManager.h"
#include "Worker.h"
#include "ResourcesManager.h"

/// Defines policy implementation structure for derived classes
/**
 * Policy class is an abstract class which is used as a template to add his own scheduling policies to
 * the application. Some function are required by derived class in order to be used as a policy.
 */
class Policy
{
protected:
	/************** MEMBERS **************/
	JobManager *jobMan;      ///< the referent Job manager
	Configuration *config;   ///< the global configuration
public:
	/************* FUNCTIONS *************/
	/**** ABSTRACTS ****/
	/// create a policy, setting class members
	Policy(JobManager* job, Configuration* config);
	/// defining how to fill a worker according to the policy
	/**
	 * The function have to fill the worker with jobs from job manager
	 * \warning Each derived class have to implement this function !
	 * \warning Do not try to take jobs from jobs list in a list where indice is larger than maxIndice !
	 * \param[in] cur the current worker to fill
	 * \param[in] maxIndice the max jobs list indice where jobs can be taken
	 * \param[in] nbCurResources current number of availables resources (used in some policies)
	 */
	virtual void fillingAlgo(Worker *cur, size_t maxIndice, size_t nbCurResources) = 0;
	///virtual destructor to unset a policy
	virtual ~Policy();
};

#endif // POLICY_H
