/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "PolicySchedRes.h"
using namespace std;

SchedResPolicy::SchedResPolicy(JobManager* jobMan, Configuration* config): Policy(jobMan, config) {

}

void SchedResPolicy::fillingAlgo(Worker * cur, size_t maxIndice,
				 size_t nbCurResources)
{
	std::list<Job*>* currentJM = jobMan->jobsLists();
	int i;
	size_t nbJobs = 0, nbMaxJobs = 0, time = 0, maxTime = 0;
	
	assert(maxIndice <= nbCurResources);
	UNUSED(nbCurResources);
	
	nbMaxJobs = config->job().getNbMaxJobs();
	i = maxIndice;
	maxTime = config->system().getMaxSlaveTime();
	
	do{
		for(list<Job*>::iterator it = currentJM[i].begin(); it != currentJM[i].end();){
			assert((*it)->getStatus() == NOT_RUN);
			if(nbJobs >= nbMaxJobs && nbMaxJobs != JobConfiguration::DEFAULT_MAX_NB_JOBS && time <= maxTime)
				break;
			else if( (*it)->isDepInvalid()){
				it++;
				continue;
			}
			time += (*it)->getExpectedTime();
			cur->add(*it);
			// here the it++ is done by erase function !!!!
			it = currentJM[i].erase(it);
			nbJobs++;
		}
	i--;
	}while(i >= 0 && cur->size() <= 0);
	// +2 = +1 for list indice and +1 for extra decrement
	cur->setNbRequiredResources(i+2);
}
