/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "PolicySchedTime.h"
using namespace std;

double SchedTimePolicy::curJobManSurface = 0.0;
int SchedTimePolicy::nbCalls = 0;

SchedTimePolicy::SchedTimePolicy(JobManager* job, Configuration* config) : Policy(job, config)
{

}

void SchedTimePolicy::fillingAlgo(Worker * cur, size_t maxIndice, size_t nbCurResources)
{

	//datas
	bool fullWorker = false;
	std::list<Job*>* currentJM = jobMan->jobsLists();
	int i;
	
	//resources
	size_t nRes_1passe = 0, nWorkers = 0, nRes = 0, biggestJob = 1;
	//surface
	size_t surface_1passe = 0, currentSurface = 0, workloadSurface = 0;
	// timers
	size_t time_1passe = 0, tWorkerMax = 0, tWorkerMin = 0, tJobMax = 0;
	
	//getters
	nRes = config->system().getNbMaxResources();
	nWorkers = config->system().getNbMaxSlaves();
	nWorkers = (nWorkers <= 0) ? 1 : nWorkers;
	//warning : if worker max not defined, value = -1 (unsigned)
	tWorkerMax = config->system().getMaxSlaveTime();
	tWorkerMin = config->system().getMinSlaveTime();
	tJobMax = config->job().getMaxJobTime();
	
	if (nbCalls%nWorkers == 0)
		SchedTimePolicy::recomputeCurSurface(jobMan->jobsLists(), jobMan->getNbLists());
	nbCalls++;
	
	workloadSurface = curJobManSurface;
	
	//before computing, some checks have to be done to ensure coherency
	assert(nRes > 0);
	assert(nWorkers > 0);
	assert(tWorkerMax >= tWorkerMin);
	assert(tJobMax <= tWorkerMax);
	
	//computing
	nRes_1passe = (int)(nRes / nWorkers);
	time_1passe = ((size_t)(workloadSurface/nRes) < tWorkerMax) ? (int)(workloadSurface/nRes) : tWorkerMax;
	
	//define min computing time for a resources (here 1 second), then all jobs are given to the next worker
	if(time_1passe < 1){
		surface_1passe = workloadSurface;
	} else {		
		surface_1passe = nRes_1passe * time_1passe;
	}

// 	cout << "nWorkers = " << nWorkers << endl;
// 	cout << "nRes = " << nRes << endl;
// 	cout << "nJobs = " << nJobs << endl;
// 	cout << "tWorkerMax = " << tWorkerMax << endl;
// 	cout << "tWorkerMin = " << tWorkerMin << endl;
// 	cout << "tJobMax = " << tJobMax << endl;
// 	cout << "workloadSurface = " << workloadSurface << endl;
// 	cout << "nRes_1passe = " << nRes_1passe << endl;
//	cout << "time_1passe = " << time_1passe << endl;
// 	cout << "surface_1passe = " << surface_1passe << endl;

	i = maxIndice;
	do{
		for(list<Job*>::iterator it = currentJM[i].begin(); it != currentJM[i].end();){
			assert((*it)->getStatus() == NOT_RUN);
			if((*it)->getExpectedTime() > tWorkerMax){
				printError("A job time is bigger than max time allowed to allocation.\n ERROR        : The \"Sched by Time\" won't schedule this job !");
			}

			// warning when handling with effective jobs time
			if(currentSurface >= surface_1passe){
				fullWorker = true;
				break;
			}
			else if( (*it)->isDepInvalid()){
				it++;
				continue;
			}
			cur->add(*it);
			currentSurface += ((*it)->getNbResources() * (*it)->getExpectedTime());
			
			if(biggestJob < (*it)->getNbResources()) 
				biggestJob = (*it)->getNbResources();
			// here the it++ is done by erase function !!!!
			it = currentJM[i].erase(it);
		}
	i--;
	}while(i >= 0 && !fullWorker);
	
	if(jobMan->isEmpty())
		nRes_1passe = nbCurResources;
	
	if((size_t)(currentSurface/nRes_1passe) < tWorkerMin)
		nRes_1passe = std::max((size_t)(currentSurface/tWorkerMin), (size_t)1);
	
	if(biggestJob > nRes_1passe)
		nRes_1passe = biggestJob;
	
	assert(nRes_1passe <= nbCurResources);
// 	cout << "FINAL = " << nRes_1passe << endl;
	cur->setNbRequiredResources(nRes_1passe);
}

void SchedTimePolicy::recomputeCurSurface(std::list<Job*>* currentJM, size_t nbLists)
{
	curJobManSurface = 0.0;
	
	for(size_t i = 0; i < nbLists ; i++){
		for(list<Job*>::iterator it = currentJM[i].begin(); it != currentJM[i].end(); it++){
			curJobManSurface += ((*it)->getExpectedTime() * (*it)->getNbResources());
		}
	}
}
