/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef SCHEDTIMEPOLICY_H
#define SCHEDTIMEPOLICY_H

#include "Policy.h"

/// Policy implementation for the "scheduling by Time" policy
/**
 * Implements specific behaviour routines
 * In this policy, jobs are grouped by the fact : "the biggest first". Thus, each
 * worker is filled with various jobs and the limit is reached when maximum
 * execution time for one worker is reached. The number of resources associated
 * to the worker depends on number of workers allowed to run at the same time.
 * \see Policy
 */
class SchedTimePolicy : public Policy
{
private:
	/************** STATICS **************/
	/**** NON-CONST ****/
	static double curJobManSurface;  ///< Current remaining working surface
	static int nbCalls;              ///< number of calls to fillingAlgo in this class

	/************* FUNCTIONS *************/
	/// recompute current working surface contained in job manger
	/**
	 * Update static variable curJobManSurface
	 * \param[in] currentJM the jobs lists
	 * \param[in] nbLists number of lists
	 */
	static void recomputeCurSurface(std::list < Job * >*currentJM, size_t nbLists);
	    
public:	
	/************* FUNCTIONS *************/
	/// create a new "scheduling by time" policy
	SchedTimePolicy(JobManager* job, Configuration* config);
	virtual void fillingAlgo(Worker * cur, size_t maxIndice,
			     size_t nbCurResources);

};

#endif // SCHEDTIMEPOLICY_H
