/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#ifndef RESOURCESMANAGER_H
#define RESOURCESMANAGER_H
#include "utils.h"

/// handles resources management and specifications according to resources handling
/**
 * The resources manager represents the machine availability over the time.
 */ 
class ResourcesManager {
private:
	/************** MEMBERS **************/
	size_t nbAvailRes;           ///< number of available resources (at any t time)
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	explicit ResourcesManager();
	/// update number of available resources
	/**
	 * the argument is added to the current number of available resources
	 * \param[in] arg1 number of resources to add to current resources manager
	 */
	void update(int arg1);
	/// defines explicitly the number of allowed resources
	/**
	 * \param[in] nb effective number of available resources
	 */
	void setNbResources(size_t nb);
	
	/****** CONST ******/
	/// get current number of resources
	/**
	 * \return number of resources
	 */
	size_t getNbResources() const;
};

#endif // RESOURCESMANAGER_H
