/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef RUNNERMASTER_H
#define RUNNERMASTER_H

#include "Runner.h"
#include "HashTable.h"

/// main class for master execution mode
/**
 * This class contains all required methods and rountines needed to master
 * specific behaviour. It's derived from Runner class and implement the whole
 * virtuals functions from this base class. 
 */
class RunnerMaster : public Runner {
private:
	/************** MEMBERS **************/
	HashTable tabWorkers;  ///< list of currently running workers
	bool firstRound;       ///< check if a first worker have been launched
	
	/************** STATICS **************/
	/****** CONST ******/
	static const size_t NB_PARAMETERS_COMMAND = 10;  ///< max number of parameters used to start a new jchronoss instance
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// behaviour used to sigaction call
	/**
	 * This function is called when an signal is caught by application. This 
	 * function is passed to sigaction call
	 * \param[in] sig the signal
	 * \param[in] siginfo info about raising
	 * \param[in] context the current signal handling context
	 */
	static void handlerSignal(int sig, siginfo_t *siginfo, void *context);
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	explicit RunnerMaster();
	/// create a RunnerMaster and set its attributes
	RunnerMaster(JobManager* jobMan, Configuration* config);
	virtual void pullJobsFromFiles();
	virtual void pushJobsIntoFiles();
	virtual void fillWorker ( Worker * cur, size_t nbResources );
	virtual Worker* waitNextWorker();
	virtual void launchWorker ( Worker* cur );
	virtual void preActions();
	virtual void postActions(Worker *cur);
	virtual void startCompilation();
};

#endif // RUNNERMASTER_H
