/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#ifndef SLAVERUNNER_H
#define SLAVERUNNER_H

#include "Runner.h"
#include "DataFlow.h"

/// main class for slave execution mode
/**
 * This class contains all required methods and rountines needed to slave
 * specific behaviour. It's derived from Runner class and implement the whole
 * virtuals functions from this base class.
 */
class RunnerSlave : public Runner {
private:
	/************** MEMBERS **************/
	pthread_mutex_t waitOnEmptySocket; ///< lock on nextWorker status
	pthread_mutex_t waitOnBusySocket;  ///< lock on nextWorker availability
	Worker* nextWorker;                ///< current worker to deal with
	FileManager* outputFile;           ///< global output file (where data will be pushed)
	FileManager* traceFile;            ///< global trace file (log for JSLoc)
	/// represent the worker object given to pthread as argument
	typedef struct sThreadItem {
		RunnerSlave* slave;   ///< the current Runner (to have lockers)
		Worker* worker;       ///< the current worker to launch
	} ThreadItem;      
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// function called by thread to launch a worker (thread function)
	/**
	 * When a worker is launched, a thread handles the worker launch and forks
	 * to track it during execution. Actually, this function wrappes launchWorkerThread
	 * for each cjob and submitting results to the runner
	 * \param[in] arg the ThreadItem pointer to pass through thread creation
	 * \return Thread exit status
	 */
	static void * launchWorkerThread(void*arg);
	/// launch a job from the worker
	/**
	 * Forks in order to start the worker on the machine
	 * \param[in] job the job to launch
	 */
	void launchWorkerThreadStart(Job* job);
	/// behaviour used to sigaction call
	/**
	 * This function is called when an signal is caught by application. This 
	 * function is passed to sigaction call
	 * \param[in] sig the signal
	 * \param[in] siginfo info about raising
	 * \param[in] context the current signal handling context
	 */
	static void handlerSignal(int sig, siginfo_t *siginfo, void *context);
public:
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	explicit RunnerSlave();
	/// Create a RunnerSlave
	RunnerSlave(JobManager* jobMan, Configuration* config);
	virtual void fillWorker ( Worker* cur , size_t nbResources);
	virtual Worker* waitNextWorker();
	virtual void launchWorker ( Worker* cur );
	virtual void preActions();
	virtual void postActions(Worker* cur);
	virtual ~RunnerSlave();
};

#endif // SLAVERUNNER_H
