/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#include "Worker.h"
using namespace std;

unsigned int Worker::numWorker = 0;

size_t Worker::getNbResources() const {
	return nbRequiredResources;
}
void Worker::setNbRequiredResources(size_t val) {
	nbRequiredResources = val;
}
FileManager* Worker::getInputFile() const {
	return inputFile;
}

FileManager* Worker::getOutputFile() const {
	return outputFile;
}

void Worker::setIOFiles(FileManager* input, FileManager* output, FileManager* trace)
{
	//input can be NULL from RunnerSlave
	assert(output != NULL);
	inputFile = input;
	outputFile = output;
	traceFile = trace;
}

void Worker::unsetIOFiles()
{
	if(traceFile != NULL) delete traceFile;
	delete outputFile;
	delete inputFile;
}

void Worker::add(Job* job)
{
	jobsList.push_back(job);
}

Worker::Worker()
{
	inputFile = NULL;
	outputFile = NULL;
	nbRequiredResources = 0;
	numWorker++;
}

Worker::~Worker()
{}

void Worker::pushJobsInputFile(int nbRemain)
{
	string command;
	ostringstream flux;
	size_t id = 1;
	CHECK(inputFile->open());
	
	flux << "<jobSuite nbRemain=\"" << (nbRemain+jobsList.size()) << "\" >\n";
	
	for(list<Job*>::iterator it = jobsList.begin(); it != jobsList.end() ; it++){
		(*it)->setId(id++);
		command = (*it)->getCommand();
		HTMLEncoding(command);
		flux << "<job>\n"
			<< "\t<name>" << (*it)->getName() << "</name>\n"
			<< "\t<deps></deps>\n"
			<< "\t<constraints></constraints>\n"
			<< "\t<resources>"<< (*it)->getNbResources()<<"</resources>\n"
			<< "\t<rc>" << (*it)->getExpectedReturn() << "</rc>\n"
			<< "\t<time>" << (*it)->getExpectedTime() << "</time>\n"
			<< "\t<command>" << command << "</command>\n"
			<< "</job>\n";
	}
	flux << "</jobSuite>";
	
	(*inputFile) << flux.str();
	CHECK(inputFile->close());
}

void Worker::pullJobsOutputFile()
{
	JobResult* resCur = NULL;
	Job* jobCur = NULL;
	std::string file = outputFile->toString();
	if(!FileManager::isCreated(&file))
		printError("An output file should exist by slave but it didn't !\n ERROR        : Have you provide scripts which do not call JCHRONOSS again ?");
	CHECK(outputFile->open());
	resCur = new JobResult;
	while(resCur->read(outputFile)){
		jobCur = NULL;
		resCur->checksum();

		for(list<Job*>::iterator it = jobsList.begin(); it != jobsList.end() ; it++){
			if((*it)->getId() == resCur->getId()){
				assert(hash_fn((*it)->getName()) == resCur->getHashName());
				jobCur = (*it);
			}
		}
		assert(jobCur != NULL);
		// some checks to do on objet read from file
		if(jobCur->getExpectedReturn() == resCur->getFinalRC())
			jobCur->updateStatus(PASSED);
		else{
			jobCur->updateStatus(FAILED);
		}
		jobCur->setResult(resCur);
		resCur = new JobResult;
	}
	delete resCur;
	CHECK(outputFile->close());
	
#ifdef NDEBUG
// 	remove worker files when worker isn't required anymore
 	remove(inputFile->toString().c_str());
	remove(outputFile->toString().c_str());
#endif
}

void Worker::pushJobsOutputFile()
{
	ostringstream trace;
	for(list<Job*>::iterator it = jobsList.begin(); it != jobsList.end() ; it++){
		JobResult cur = (*it)->getResult();
		cur.write(outputFile);
#ifdef ENABLE_TRACE
		trace << (*it)->getName() << fixed << setprecision(2) << ":" << cur.getStartTime() << ":" << (cur.getStartTime() + cur.getTime()) << ":" << (*it)->getNbResources() << endl;  
		(*traceFile) << trace.str();
#endif
	}
}

std::list< Job* >::iterator Worker::begin() {
	return jobsList.begin();
}

std::list< Job* >::iterator Worker::end() {
	return jobsList.end();
}

void Worker::display() const {
	cout 	<< "+------------------ WORKER -----------------+" << endl;
	cout	<< "| - NbResources = " << nbRequiredResources << endl;
	cout	<< "+-------------------------------------------+" << endl;
	for(list<Job*>::const_iterator it = jobsList.begin(); it != jobsList.end(); it++)
		(*it)->display();
	cout	<< "+-------------------------------------------+" << endl;
}

size_t Worker::size() const
{
	return jobsList.size();
}
