/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "XMLConfigParser.h"
using namespace std;

XMLConfigParser::XMLConfigParser(std::string fileName)
{
	nameFile = new FileManager(&fileName, INPUT_ACCESS);
	document = xmlParseFile(fileName.c_str());
	if (document == NULL) {
		printError("XML file '"+fileName+"' not found !\n");
		abort();
	}
	checkConfig();
	
	jobNode = findChildNode(xmlDocGetRootElement(document), (char*)"job");
	systemNode = findChildNode(xmlDocGetRootElement(document), (char*)"system");
}

void XMLConfigParser::checkConfig()
{
	XMLValidator validator;
	
	if(!validator.validate(document, XMLValidator::xsdConfigValidation)){
		printError("XML Config file '"+nameFile->toString()+"' not valid !\n");
	}
}

void XMLConfigParser::loadInt(int &val, const bool parent, const char nodeName[]) const
{
	xmlNodePtr dad = (parent == JOB_CONF) ? jobNode : systemNode;
	string chain = findChildNodeContent(dad, (char*)nodeName);
	if(chain != "")
		val =  atoi(chain.c_str());
}
void XMLConfigParser::loadBool(bool& val, const bool parent, const char nodeName[]) const{
	xmlNodePtr dad = (parent == JOB_CONF) ? jobNode : systemNode;
	string chain = findChildNodeContent(dad, (char*)nodeName);
	if(chain != ""){
		for(size_t i=0; i < chain.size(); i++) chain[i] = tolower(chain[i]);
		val = (chain == "true");
	}
}
void XMLConfigParser::loadList(std::vector< FileManager* >& list, const bool parent, const char globalNodeName[]) const
{
	xmlNodePtr dad = (parent == JOB_CONF) ? jobNode : systemNode;
	xmlNodePtr child = findChildNode(dad, (char*)globalNodeName);
	child = findChildNode(child, (char*)"item");
	while(child != NULL){
		list.push_back(new FileManager(new string(getNodeContent(child)), INPUT_ACCESS));
		child = getNextSiblingNode(child);
	}
}
void XMLConfigParser::loadStr(std::string& val, const bool parent, const char nodeName[]) const
{
	xmlNodePtr dad = (parent == JOB_CONF) ? jobNode : systemNode;
	string chain = findChildNodeContent(dad, (char*)nodeName);
	if(chain != "")
		val =  chain;
}

XMLConfigParser::~XMLConfigParser()
{
	delete nameFile;
	xmlFreeDoc(document);
}
