/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "XMLParser.h"
using namespace std;

xmlNodePtr XMLParser::findChildNode ( xmlNodePtr parent, char* tagname ) const {
	xmlNodePtr child = NULL;

	/* checks */
	assert(parent);
	assert(tagname != NULL);

	child = getNextChildNode(parent);
	if(child != NULL){	
		if(xmlStrcmp(child->name,(const xmlChar*)tagname) == 0 )
			return(child);
		child = findSiblingNode(child, tagname);
	}
	return child;
}


xmlNodePtr XMLParser::findSiblingNode ( xmlNodePtr node, char* tagname ) const {
	xmlNodePtr sibling = NULL;

	/* checks */
	assert(node != NULL);
	assert(tagname != NULL);

	sibling = getNextSiblingNode(node);
	
	while(sibling != NULL){
		if(xmlStrcmp(sibling->name, (const xmlChar*)tagname) == 0 ){
			return(sibling);
		}
		sibling = getNextSiblingNode(sibling);
	}

	return sibling;
}


xmlNodePtr XMLParser::getNextChildNode ( xmlNodePtr parent ) const {
	
	xmlNodePtr cur = NULL;
	assert(parent != NULL);

	if (parent == NULL)
		return(NULL);

	if(!checkValidNode(parent))
		return(NULL);
	
	cur = parent->children;

	while (cur != NULL) {
		if (cur->type == XML_ELEMENT_NODE)
			return(cur);
		cur = cur->next;
	}
	return(NULL);
}


xmlNodePtr XMLParser::getNextSiblingNode ( xmlNodePtr node ) const {
	if (node == NULL)
		return(NULL);

	if(! checkValidNode(node))
		return(NULL);
	
	node=node->next;

	while (node != NULL) {
		if (node->type == XML_ELEMENT_NODE)
			return(node);
		node = node->next;
	}

	return(NULL);
}


string XMLParser::findChildNodeContent ( xmlNodePtr node, char* tagname ) const {
	assert(node != NULL);
	assert(tagname != NULL);

	node = findChildNode(node, tagname);
	if(node == NULL) return "";
	return(getNodeContent(node));
}


string XMLParser::findSiblingNodeContent ( xmlNodePtr node, char* tagname ) const {
	assert(node != NULL);
	assert(tagname != NULL);
	
	node = findSiblingNode(node, tagname);
	if(node == NULL) return "";
	return(getNodeContent(node));
}


string XMLParser::getNodeContent ( xmlNodePtr cur ) const {
	char * tmpResult = NULL;
	string result = "";
	if(cur == NULL) return (result);
	
	tmpResult = (char*)xmlNodeGetContent(cur);
	assert(tmpResult != NULL);
	
	result = tmpResult;
	free(tmpResult);
	
	return(result);
}


bool XMLParser::checkNodeExists ( xmlNodePtr node, char* name ) const {
	assert(name != NULL);

	if(xmlStrcmp(node->name, (const xmlChar*)name) != 0){
		printError("Node "+string(name)+" not found in one of XML File ! ("+string((char*)node->name)+")"); 
		abort();
	}
	return true;
}


bool XMLParser::checkValidNode ( xmlNodePtr node ) const {
	switch (node->type) {
		case XML_ELEMENT_NODE:
		case XML_TEXT_NODE:
		case XML_CDATA_SECTION_NODE:
		case XML_ENTITY_REF_NODE:
		case XML_ENTITY_NODE:
		case XML_PI_NODE:
		case XML_COMMENT_NODE:
		case XML_DTD_NODE:
		case XML_XINCLUDE_START:
		case XML_XINCLUDE_END:
			return true;
		default:
			return false;
	}
}

void XMLParser::cleanUp()
{
	xmlCleanupParser();
}
