/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef XMLPARSER_H
#define XMLPARSER_H

#include "utils.h"
#include "FileManager.h"
#include "XMLValidator.h"

/// Interface between XML and application
/**
 * This class implements routines to manipulate XML files in order to catch each node in
 * DOM Tree easily.
 */
class XMLParser {
protected:
	/************** MEMBERS **************/
	FileManager* nameFile; ///< a pointer on real file
	xmlDocPtr document;    ///< the referent document (libxml type)
	
public:
	/************** STATICS **************/
	/// Clean LibXML allocations
	/**
	 * This function is a routine for xmlCleanupParser() call. It should be called
	 * once because this function deletes all memory allocation at the same time, even
	 * if multiple threads have used the library.
	 * \warning this function is not thread safe. can only be called in sequential code
	 */
	static void cleanUp();
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///find a child node from parent in XML tree.
	/**
	 * @param parent the node where research is started 
	 * @param tagname node pattern we are looking for
	 * @return a pointer on founded node, NULL otherwise
	 */
	xmlNodePtr findChildNode(xmlNodePtr parent, char *tagname) const;
	///find a sibling node, (horizonal research) in XML tree
	/**
	 * @param node level research
	 * @param tagname node name we looking for
	 * @return a pointer on founded node, NULL otherwise
	 */
	xmlNodePtr findSiblingNode(xmlNodePtr node, char *tagname) const;
	///get contents of child node
	/**
	 * @param parent parent node
	 * @return a pointer on founded node, NULL otherwise
	 */
	xmlNodePtr getNextChildNode(xmlNodePtr parent) const;
	/// get contents of sibling node
	/** 
	 * @param node current node
	 * @return a pointer on founded node, NULL otherwise
	 */
	xmlNodePtr getNextSiblingNode(xmlNodePtr node) const;
	///Find a child node matching with pattern and return its contents 
	/**
	 * @param node the node where the research is launched
	 * @param tagname the node pattern to match
	 * @return node contents, NULL otherwise
	 */
	std::string findChildNodeContent(xmlNodePtr node, char *tagname) const;
	///Find a sibling node matching with pattern and return its contents 
	/**
	 * @param node the node where the research is launched
	 * @param tagname the node pattern to match
	 * @return node contents, NULL otherwise
	 */
	std::string findSiblingNodeContent(xmlNodePtr node, char *tagname) const;
	///get content of a node
	/** 
	 * @param cur node where data are getted
	 * @return node content, NULL otherwise
	 */
	std::string getNodeContent(xmlNodePtr cur) const;
	///Controls if specific node exists in xml File
	/**
	 * @param node node to control
	 * @param name pattern wich have to match with node name
	 * @warning his functions aborts if pattern doesn't match
	 * \return <b>True</b> if node have the same name as "name"
	 * \return <b>False</b> otherwise
	 */
	bool checkNodeExists(xmlNodePtr node, char *name) const;
	///Check if current node is a valid one
	/** 
	 * @param node node to control
	 * @return 1 if valid, 0 otherwise
	 */
	bool checkValidNode(xmlNodePtr node) const;
};

#endif // XMLPARSER_H
