/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "XMLValidator.h"
using namespace std;

bool XMLValidator::validate(xmlDocPtr doc, std::string xsd) {

	bool ret = false;
	this->pctxt = xmlSchemaNewMemParserCtxt(xsd.c_str(), xsd.size());
	this->schema = xmlSchemaParse(pctxt);
	xmlSchemaFreeParserCtxt(pctxt);
	this->vctxt = xmlSchemaNewValidCtxt(this->schema);

 	ret = (xmlSchemaValidateDoc(this->vctxt, doc) == 0 ) ? true : false;
	xmlSchemaFree(schema);
	xmlSchemaFreeValidCtxt(vctxt);
	
	
	return ret;
}

const string XMLValidator::xsdJobsvalidation = " \
<!--<?xml version=\"1.0\" encoding=\"UTF-8\"?> -->\
<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\"> \
        <xs:element name=\"name\" type=\"xs:string\"/> \
        <xs:element name=\"command\" type=\"xs:string\"/>\
        <xs:element name=\"rc\" type=\"xs:string\"/>\
        <xs:element name=\"time\" type=\"xs:decimal\"/>\
        <xs:element name=\"resources\" type=\"xs:string\"/>\
        <xs:element name=\"dep\" type=\"xs:string\"/>\
        <xs:element name=\"constraint\" type=\"xs:string\"/>\
        <xs:element name=\"deps\">\
                <xs:complexType>\
                        <xs:sequence>\
                                <xs:element ref=\"dep\" minOccurs=\"0\" maxOccurs=\"unbounded\"/>\
                        </xs:sequence>\
                </xs:complexType>\
        </xs:element>\
        <xs:element name=\"constraints\">\
                <xs:complexType>\
                        <xs:sequence>\
                                <xs:element ref=\"constraint\" minOccurs=\"0\" maxOccurs=\"unbounded\"/>\
                        </xs:sequence>\
                </xs:complexType>\
        </xs:element>\
        <xs:element name=\"job\">\
                <xs:complexType>\
                        <xs:all>\
                                <xs:element ref=\"name\" minOccurs=\"1\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"command\" minOccurs=\"1\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"rc\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"time\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"resources\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"deps\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"constraints\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                        </xs:all>\
                </xs:complexType>\
        </xs:element>\
        <xs:element name=\"jobSuite\">\
                <xs:complexType>\
                        <xs:sequence>\
                                <xs:element ref=\"job\" minOccurs=\"0\" maxOccurs=\"unbounded\"/>\
                        </xs:sequence>\
                        <xs:attribute name=\"package\" type=\"xs:string\" use=\"optional\"/>\
                        <xs:attribute name=\"nbRemain\" type=\"xs:string\" use=\"optional\"/>\
                </xs:complexType>\
        </xs:element>\
</xs:schema>";

const string XMLValidator::xsdConfigValidation = "\
<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\
<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">\
        <xs:element name=\"item\" type=\"xs:string\"/>\
        <xs:element name=\"verbosity\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"logging\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"fakeExecution\" type=\"xs:string\"/>\
        <xs:element name=\"maxJobTime\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"maxNbJobs\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"jobsCommand\" type=\"xs:string\"/>\
        <xs:element name=\"compilationCommand\" type=\"xs:string\"/>\
        <xs:element name=\"output\" type=\"xs:string\"/>\
        <xs:element name=\"build\" type=\"xs:string\"/>\
        <xs:element name=\"maxResources\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"maxSlaves\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"maxSlaveTime\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"minSlaveTime\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"autokill\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"policy\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"flowsize\" type=\"xs:nonNegativeInteger\"/>\
        <xs:element name=\"jobslist\">\
                <xs:complexType>\
                        <xs:sequence>\
                                <xs:element ref=\"item\" minOccurs=\"0\" maxOccurs=\"unbounded\"/>\
                        </xs:sequence>\
                </xs:complexType>\
        </xs:element>\
        <xs:element name=\"whitelist\">\
                <xs:complexType>\
                        <xs:sequence>\
                                <xs:element ref=\"item\" minOccurs=\"0\" maxOccurs=\"unbounded\"/>\
                        </xs:sequence>\
                </xs:complexType>\
        </xs:element>\
        <xs:element name=\"blacklist\">\
                <xs:complexType>\
                        <xs:sequence>\
                                <xs:element ref=\"item\" minOccurs=\"0\" maxOccurs=\"unbounded\"/>\
                        </xs:sequence>\
                </xs:complexType>\
        </xs:element>\
        <xs:element name=\"job\">\
                <xs:complexType>\
                        <xs:all>\
                                <xs:element ref=\"jobslist\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"whitelist\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"blacklist\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"verbosity\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"logging\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"fakeExecution\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"maxJobTime\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"maxNbJobs\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                        </xs:all>\
                </xs:complexType>\
        </xs:element>\
	<xs:element name=\"system\">\
                <xs:complexType>\
                        <xs:all>\
                                <xs:element ref=\"jobsCommand\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"compilationCommand\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"output\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"build\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"maxResources\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"maxSlaves\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"maxSlaveTime\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"minSlaveTime\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"policy\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"flowsize\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"autokill\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                        </xs:all>\
                </xs:complexType>\
        </xs:element>\
        <xs:element name=\"configuration\">\
                <xs:complexType>\
                        <xs:all>\
                                <xs:element ref=\"job\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                                <xs:element ref=\"system\" minOccurs=\"0\" maxOccurs=\"1\"/>\
                        </xs:all>\
                </xs:complexType>\
        </xs:element>\
</xs:schema>";
