/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef XMLVALIDATOR_H
#define XMLVALIDATOR_H

#include "utils.h"
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xmlschemas.h>

/// validate a XML file with a XML scheme
/**
 * This class is used to ensure data are well-formatted
 */
class XMLValidator {
private:
	/************** MEMBERS **************/
	xmlSchemaPtr schema;          ///< the schema pointer on the doc
	xmlSchemaValidCtxtPtr vctxt;  ///< the xml validator
	xmlSchemaParserCtxtPtr pctxt; ///< the schema loader

public:
	/************** STATICS **************/
	/****** CONST ******/	
	static const std::string xsdJobsvalidation; ///< the xml schema in string format
	static const std::string xsdConfigValidation; ///< the xml schema in string format for configuration file validation
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// apply xml validation to the given xml document
	/**
	 * The xml schema is applied to the given document in order to ensure well-formatted
	 * data
	 * \param[in] doc the xml document to validate
	 * \param[in] xsd the xml schema to use (in statics one)
	 * \return <b>True</b> if document is found, opened and valid
	 * \return <b>False</b> otherwise
	 */
	bool validate(xmlDocPtr doc, std::string xsd);
};

#endif // XMLVALIDATOR_H
