/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "utils.h"
using namespace std;

double getCurrentDate() {
	double time=0.0;
	struct timeval tic;

	gettimeofday(&tic, NULL);

	time = (double)tic.tv_sec+ (double)(tic.tv_usec/1000000.0);
	return time;
}

void printLine ( string prefix, string format, ... ) {
	va_list args;
	int cursor=0;
	char output[1024];

	cursor += sprintf(output, "%s", prefix.c_str());
	va_start(args, format);

	cursor += vsprintf(output+cursor, format.c_str(), args);
	cursor += sprintf(output+cursor, "%s", (char*)COLOR_NORM);

	va_end(args);
	write(1, output, cursor);
}

int safeRead(FILE* fd, void * buf, int size) {
	int total_count=0, current_count=0;
	
	/* reading while not all the characters are been read */
	while(total_count < size){
		current_count=fread((char*)buf+total_count, sizeof(char), size - total_count, fd);
		/* if there are nothing to read anymore */
		if(current_count == 0){
			break;
			/* error case */
		}else if( current_count < 0){
			/* if signal interruption, reading continuation */
			if(errno == EINTR) continue;
			/* aborting if others errors */
			else{
				printf("Error reading %p %d/%d\n", buf, total_count, size);
				exit(1);
			}
		}
		total_count += current_count;
	}
	return total_count;
}

bool matchWith (std::string chain,std::string prefix ) {
	return (chain.compare(0, prefix.size(), prefix, 0, prefix.size()) == 0);
}

std::string* splitArgumentPrefix ( std::string chain, std::string prefix ) {
	chain.erase(0, prefix.size());
	return new string(chain);
}

void HTMLEncoding(string& src){
	string buffer;
	buffer.reserve(src.size());
	for(std::string::size_type i = 0 ; i != src.size(); i++)
	{
		switch(src[i]){
			case '&':  buffer.append("&amp;"); break;
			case '\"': buffer.append("&quot;"); break;
			case '\'': buffer.append("&apos;"); break;
			case '<':  buffer.append("&lt;"); break;
			case '>':  buffer.append("&gt;"); break;
// 			case 'é':  buffer.append("&eacute;"); break;
// 			case 'è':  buffer.append("&egrave;"); break;
// 			case 'à':  buffer.append("&agrave;"); break;
			default :  buffer.append(&src[i], 1); break;
		}
	}
	
	src.swap(buffer);
}

size_t hash_fn(string toHash){
	size_t value = 0;
	for(size_t i=0; i<toHash.size(); i++)
		value += (int)toHash[i]*i;
	return value;
}

void printHeader() {
	cout << "+--------------------------------------------------------------+" << endl;
	cout << "|   STATUS    |   ID  /  SZ   | LEFT |    TIME    |    NAME    |" << endl;
	cout << "+--------------------------------------------------------------+" << endl;
}

void banner(){
#ifdef ENABLE_COLOR
	cout	<<COLOR_FAIL"    ___ "COLOR_URUN" _____  _   _ "COLOR_PASS"______  _____  _   _  _____  _____        \n"
		<<COLOR_FAIL"   |_  |"COLOR_URUN"/  __ \\| | | |"COLOR_PASS"| ___ \\|  _  || \\ | ||  _  |/  ___|       \n"
		<<COLOR_FAIL"     | |"COLOR_URUN"| /  \\/| |_| |"COLOR_PASS"| |_/ /| | | ||  \\| || | | |\\ `--.        \n"
		<<COLOR_FAIL"     | |"COLOR_URUN"| |    |  _  |"COLOR_PASS"|    / | | | || . ` || | | | `--. \\_____  \n"
		<<COLOR_FAIL" /\\__/ /"COLOR_URUN"| \\__/\\| | | |"COLOR_PASS"| |\\ \\ \\ \\_/ /| |\\  |\\ \\_/ //\\__/ /  ___| \n"
		<<COLOR_FAIL" \\____/ "COLOR_URUN" \\____/\\_| |_/"COLOR_PASS"\\_| \\_| \\___/ \\_| \\_/ \\___/ \\____/\\ `--.  \n"
		<<COLOR_FAIL"        "COLOR_URUN"              "COLOR_PASS"                                   `--. \\ \n"
		<<COLOR_FAIL"        "COLOR_URUN"              "COLOR_PASS"                                  /\\__/ / \n"
		<<COLOR_FAIL"        "COLOR_URUN"              "COLOR_PASS"                                  \\____/  \n"
		<<COLOR_NORM"\n";

#else
	cout	<<"    ___  _____  _   _ ______  _____  _   _  _____  _____        \n"
		<<"   |_  |/  __ \\| | | || ___ \\|  _  || \\ | ||  _  |/  ___|       \n"
		<<"     | || /  \\/| |_| || |_/ /| | | ||  \\| || | | |\\ `--.        \n"
		<<"     | || |    |  _  ||    / | | | || . ` || | | | `--. \\_____  \n"
		<<" /\\__/ /| \\__/\\| | | || |\\ \\ \\ \\_/ /| |\\  |\\ \\_/ //\\__/ /  ___| \n"
		<<" \\____/  \\____/\\_| |_/\\_| \\_| \\___/ \\_| \\_/ \\___/ \\____/\\ `--.  \n"
		<<"                                                         `--. \\ \n"
		<<"                                                        /\\__/ / \n"
		<<"                                                        \\____/  \n"
		<<"\n";
#endif
}
