#!/bin/sh

############################################################################
#                                                                          #
# Tue Jul 22 13:28:10 CEST 2014                                            #
# Copyright or (C) or Copr. Commissariat a l'Energie Atomique              #
#                                                                          #
# This file is part of JCHRONOSS                                           #
#                                                                          #
# JCHRONOSS is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU Lesser General Public License as           #
# published by the Free Software Foundation, either version 3 of the       #
# License, or (at your option) any later version.                          #
#                                                                          #
# JCHRONOSS is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of           #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
# GNU Lesser General Public License for more details.                      #
#                                                                          #
# You should have received a copy of the GNU Lesser General Public License #
# along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                          #
# Version : 1.0                                                            #
# Author  : Julien Adam <julien.adam@cea.fr>                               #
#                                                                          #
############################################################################

SOURCES_LIST=""
WEBVIEW_PATH="."
NEW_PATH=""
DIFF_PATH=""

###################### FUNCTION ####################
# Args :
#  - output file
#  - command
generate()
{
	output=$1
	echo "  - Generate `basename $output`"
	shift
	$* > $output || exit 1
}

###################### FUNCTION ####################
# Args :
#  - directory
cleanup()
{
	echo " * Cleanup $1" 
	if [ ! -z "$1" ]; then
		rm -f $1/* || exit 1
	fi
}

###################### FUNCTION ####################
# Args:
#  - output
#  - xslt
#  - xml
generate_by_xsl()
{
	output=$1
	echo "  - Generate `basename $1`"
	shift
	#echo xsltproc $*
	xsltproc $* > $output || exit 1
}

############## CHECK SOURCES #####################
#
#
check_sources(){
	
	if [ -z "$NEW_PATH" ] ; then
		echo "You have to provide a valid path where output files are stored"
		exit 1
	fi

	SOURCES_LIST="$NEW_PATH"    
}

check_reference(){

	if [ -z "$DIFF_PATH" ]  ; then
		echo "  - No reference data submitted (or not found) : Disabling Diff capabilities"
		HAVE_REFERENCE="no"
	else HAVE_REFERENCE="yes"
	fi
}

#check if a file in $DIFF_TARBALL match with $1 file
file_matching(){
	
	file_ref="./reference/last_results/$(echo "$1" | awk -F'^./' '{print $2}' 2> /dev/null)"
	
	echo $file_ref

	if [ -f $file_ref ] ; then
		return 0
	fi
	return 1
}

print_help(){
	echo "################################## WEBVIEW USAGE #############################"
	echo "                ./webview_gen_all.sh --new=<path> [ --old=<path> ]           #"
	echo "##############################################################################"
	echo "#  --new=<path>   Specifies an input folder (current validation)             #"
	echo "#  --old=<path>   Specifies a reference folder (previous validation)         #"
	echo "##############################################################################"
}
######################## MAIN ######################

if [ ! -f "${PWD}/webview_gen_all.sh" ] ; then
	WEBVIEW_PATH="`dirname $0`"
fi

## BANNER ##
cat ${WEBVIEW_PATH}/webview/webview_banner

for arg in $*
do
	case $arg in
		--new=*)
			NEW_PATH="$(echo "$arg" | sed -e "s@^--new=@@g")"
		;;
		--old=*)
			DIFF_PATH="$(echo "$arg" | sed -e "s@^--old=@@g")"
		;;
		-h|--help)
			print_help
			exit 0
		;;
		*)
			echo "Ignored : --> $arg"
		;;
	esac
done

echo " * Check sources"
check_sources
echo " * Check reference sources"
check_reference

#cleanup of mkdir
if [ -d ${WEBVIEW_PATH}/webview/generated ]; then
	cleanup ${WEBVIEW_PATH}/webview/generated
else
	echo "create webview/generated"
	mkdir ${WEBVIEW_PATH}/webview/generated || exit 1
fi

export SOURCES_LIST
export WEBVIEW_PATH

echo " * Start generation :"
#generate summary
generate ${WEBVIEW_PATH}/webview/generated/main.xml ${WEBVIEW_PATH}/webview/scripts/webview_gen_global_stats.sh
generate ${WEBVIEW_PATH}/webview/generated/errors.xml ${WEBVIEW_PATH}/webview/scripts/webview_gen_fail_list.sh

#generate html pages
generate_by_xsl ${WEBVIEW_PATH}/webview/generated/main.html ${WEBVIEW_PATH}/webview/generators/main.html.xsl ${WEBVIEW_PATH}/webview/generated/main.xml
generate_by_xsl ${WEBVIEW_PATH}/webview/generated/errors.html ${WEBVIEW_PATH}/webview/generators/errors.html.xsl ${WEBVIEW_PATH}/webview/generated/errors.xml

for file in $(find $SOURCES_LIST -iname 'output*.xml' )
do
	out_filename="$(egrep -o "<testsuite name=\"[^\"]+\"" $file | sed -e "s@<testsuite name\=@@g" -e "s@\"@@g" -e 's#\.#-#g' -e "s#/#-#g").html"
	generate_by_xsl ${WEBVIEW_PATH}/webview/generated/$out_filename ${WEBVIEW_PATH}/webview/generators/detail.html.xsl $file

	diff_out_filename="$(egrep -o "<testsuite name=\"[^\"]+\"" $file | sed -e "s@<testsuite name\=@@g" -e "s@\"@@g" -e 's#\.#-#g' -e "s#/#-#g").xml"
	if [ "${HAVE_REFERENCE}" = "yes" ] ; then
		file_ref="`file_matching "$file"`"
		if [ $? -eq  0 ] ; then
			generate ${WEBVIEW_PATH}/webview/generated/diff-$diff_out_filename ${WEBVIEW_PATH}/webview/scripts/webview_gen_diff.sh $file $file_ref
			rm -f temp.xml
			generate_by_xsl ${WEBVIEW_PATH}/webview/generated/diff-computed-$diff_out_filename ${WEBVIEW_PATH}/webview/generators/diff-compute.xml.xsl ${WEBVIEW_PATH}/webview/generated/diff-$diff_out_filename
			generate_by_xsl ${WEBVIEW_PATH}/webview/generated/diff-$out_filename ${WEBVIEW_PATH}/webview/generators/diff.html.xsl ${WEBVIEW_PATH}/webview/generated/diff-computed-$diff_out_filename
		
		else
			generate_by_xsl ${WEBVIEW_PATH}/webview/generated/diff-$out_filename ${WEBVIEW_PATH}/webview/generators/diff-file-not-found.html.xsl $file 
			echo "    /!\ WARNING: file $file_ref NOT FOUND, Diff not generated"
		fi
	else
		generate_by_xsl ${WEBVIEW_PATH}/webview/generated/diff-$out_filename ${WEBVIEW_PATH}/webview/generators/diff-file-not-found.html.xsl $file 
	fi
done

#finish
echo ""
###################### FINISH ######################
echo "You can browse by opening $WEBVIEW_PATH/webview/index.html in your browser"
echo "or manually start a small web server : cd webview && python -m SimpleHTTPServer"
